#include "tcpip.h"

struct realm *tcp_realm;

unsigned int num_tcp_head=0;
unsigned int num_tcp_rules=0;

/* tcp_rules_arr[state][dir] */
struct tcp_head *tcp_rules_arr[3][3]={
	{NULL, NULL, NULL},
	{NULL, NULL, NULL},
	{NULL, NULL, NULL}
};

char *tcp_ignore_words[]={
	"stateless",
	"flow",
	NULL
};

/* Generator */
struct generator tcp_gen=init_generator("sig.tcp");

int tcp_do_match(struct tcp_head *h, struct packet *p, unsigned int l)
{
	struct pkt_iphdr *iph;
	struct pkt_tcphdr *tcph;
	u_int16_t sp, dp;

	iph=p->layer[l].h.ip;
	tcph=p->layer[l+1].h.tcp;
	sp=ntohs(tcph->sport);
	dp=ntohs(tcph->dport);

	for(; h; h=h->next) {
		if ( 
			(((h->saddr&h->smask) == (iph->saddr&h->smask))
			^ ((h->flags&NEGATE_SADDR)!=0) ) &&
			
			(((h->daddr&h->dmask) == (iph->daddr&h->dmask))
			^ ((h->flags&NEGATE_DADDR)!=0) ) &&

			(((sp>=h->sport_min)&&(sp<=h->sport_max)))
			^ ((h->flags&NEGATE_SPORT)!=0) &&
			
			(((dp>=h->dport_min)&&(dp<=h->dport_max)))
			^ ((h->flags&NEGATE_DPORT)!=0)){

			if (tcpip_traverse(&tcp_gen, p, l, h->rules))
				return 1;
		}
	}

	return 0;
}

void tcp_match(struct packet *p, unsigned int l)
{
	int i,j;
	int state=0;
	int direction=2;
	unsigned int flags=p->layer[l+1].flags;

	if ( !(flags&FLAG_TCP_TRACK) ) {
		goto do_all;
	}

	if ( (flags&FLAG_TCP_STATE) ) {
		state=1;
		if ( flags & FLAG_TCP_SURE ) {
			state=2;
		}
	}

	if ( (flags&FLAG_TCP_2SVR) )
		direction=1;

	for(i=0; i<=state; i++) {
		if ( tcp_do_match(tcp_rules_arr[i][direction], p, l)>0 )
			return;
		
		if ( tcp_do_match(tcp_rules_arr[i][0], p, l)>0 )
			return;
	}

	return;

do_all:
	for(i=0; i<3; i++) {
		for(j=0; j<3; j++) {
			if ( tcp_do_match(tcp_rules_arr[i][j], p, l)>0 )
				return;
		}
	}
}

int tcp_flow(char *sstr, int *sf, int *df)
{
	char str[128];
	char *cur, *tok;
	int end=1;
	
	if ( snprintf(str, sizeof(str), "%s", sstr)<0 )
		return 0;

	for(tok=cur=str; end; cur++) {
		if ( *cur==',' || *cur==0 ) {
			if ( *cur==0 ) end=0;
			*cur=0;
			if ( !strcmp(tok, "to_server") ||
				!strcmp(tok, "from_client")) {
				*df=1;
			}else if ( !strcmp(tok, "to_client") ||
				!strcmp(tok, "from_server") ) {
				*df=2;
			}else if ( !strcmp(tok, "stateless") ) {
				*sf=1;
			}else if ( !strcmp(tok, "established") ) {
				*sf=2;
			}else{
				printf("flow: warning: '%s' "
					"not supported\n", tok);
			}
			tok=cur+1;
		}
	}
	
	return 1;
}

int tcp_commit(struct rule *r)
{
	struct tcp_head **tcph;
	struct criteria *c;
	int s_flags=1; /* stateless, stateful, established */
	int d_flags=0; /* both, to_server, to_client */

	/* Stateful by default. */
	for(c=r->criteria; c; c=c->next) {
		if ( !strcmp("stateless", c->crit) ) {
			s_flags=0;
		}else if ( !strcmp("flow", c->crit) ) {
			if ( !tcp_flow(c->args, &s_flags, &d_flags) )
				return 0;
		}
	}

	tcph=&tcp_rules_arr[s_flags][d_flags];

	return tcp_udp_commit(tcph,
		&num_tcp_head,
		&num_tcp_rules, r,
		tcp_ignore_words);
}

struct proto tcp_p=init_proto_ex("tcp", tcp_decode, tcp_dprint);
struct proto_req tcp_r[]={
	proto_request("ipv4", 6),
	null_request()
};

int tcp_dprint(struct layer *l, char *buf, int buflen)
{
	char tcpflags[]="FSRPAU12";
	char disp[9];
	int n, x;
	
	for(n=0,x=1; n<8; n++, x*=2) {
		if ( l->h.tcp->flags.flags & x ){
			disp[n]=tcpflags[n];
		}else{
			disp[n]='*';
		}
	}disp[8]='\0';
	
	return snprintf(buf, buflen,
		"%u > %u [%s] seq=0x%x ack=0x%x win=%u",
		ntohs(l->h.tcp->sport),
		ntohs(l->h.tcp->dport),
		disp,
		ntohl(l->h.tcp->seq),
		ntohl(l->h.tcp->ack),
		ntohs(l->h.tcp->win));
}

int tcp_decode(struct packet *p)
{
	struct proto_child *pc;
	struct layer *l=&p->layer[p->llen-1];
	int ret=0;

	if ( p->llen >= PKT_LAYERS ) return ret;

	/* Check it is at least minimum size */
	if ( l->h.raw+sizeof(struct pkt_tcphdr) >= p->end )
		return ret;
	
	/* Use the doff field to decode the packet */
	if ( (p->layer[p->llen].h.raw=
		l->h.raw+(l->h.tcp->doff<<2))
		>= p->end ) {
		p->layer[p->llen].h.raw=p->end;
		return ret;
	}

	for(pc=l->proto->children; pc; pc=pc->next)
	{
		if ( l->h.tcp->dport == pc->id ||
			l->h.tcp->sport == pc->id ) {
			p->layer[p->llen].flags=0;
			p->layer[p->llen].session=NULL;
			p->layer[p->llen++].proto=pc->proto;
			return pc->proto->decode(p) | ret;
		}
	}

	p->layer[p->llen++].proto=NULL;
	return ret;
}
