/*
 * This file is part of firestorm NIDS
 * Copyright (c) 2002 Gianni Tedesco
 */

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <netinet/in.h>

#include <packet.h>
#include <signature.h>
#include <matcher.h>
#include <plugin.h>

#define DSM_EQ	0
#define DSM_LT	1
#define DSM_GT	2

struct ds_priv {
	unsigned int	dsize;
};

/* dsize matcher: match packet payload size */
int ds_match_eq(struct packet *p, void *priv, unsigned int l, int n)
{
	struct ds_priv *dsp=priv;
	unsigned int dsize;
	if ( p->layer[p->llen-1].proto ) return n^0;
	dsize=p->end - p->layer[p->llen-1].h.raw;
	return n ^ (dsize==dsp->dsize);
}

int ds_match_gt(struct packet *p, void *priv, unsigned int l, int n)
{
	struct ds_priv *dsp=priv;
	unsigned int dsize;
	if ( p->layer[p->llen-1].proto ) return n^0;
	dsize=p->end - p->layer[p->llen-1].h.raw;
	return n ^ (dsize>dsp->dsize);
}

int ds_match_lt(struct packet *p, void *priv, unsigned int l, int n)
{
	struct ds_priv *dsp=priv;
	unsigned int dsize;
	if ( p->layer[p->llen-1].proto ) return n^0;
	dsize=p->end - p->layer[p->llen-1].h.raw;
	return n ^ (dsize<dsp->dsize);
}

proc_match_match ds_validate(char *args, void **priv, struct criteria *m)
{
	struct ds_priv *dsp;
	proc_match_match ret=ds_match_eq;
	unsigned int num;
	char *val;

	if ( !args ) return NULL;
	
	for(val=args; *val; val++) {
		if ( *val>='0' && *val<='9' ) {
			break;	
		}
		if ( *val=='<' ) {
			ret=ds_match_lt;
		}else if ( *val=='>' ){
			ret=ds_match_gt;
		}
	}
	
	if ( (num=strtoul(val, NULL, 0))==ULONG_MAX &&
		errno==ERANGE ) return NULL;

	if ( !(dsp=malloc(sizeof(*dsp))) )
		return NULL;

	dsp->dsize=num;
	*priv=dsp;

	return ret;
}

int _PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	int ok=0;
	proc_matcher_add matcher_add;
	
	plugin_check(in, out);
	
	PLUGIN_ID("match.std", "Generic matching routines");
	PLUGIN_VERSION(2, 0);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@scaramanga.co.uk");
	PLUGIN_LICENSE("GPL");

	if ( !(matcher_add=in->import("matcher.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	ok+=matcher_add("dsize",
		ds_validate,
		NULL,
		in->import("template.ptr"));
	
	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int _PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
